/* Map2Obj.h
    implementions for Map2Obj.c */

#include <stdio.h>

/* Definations */
#define uchar unsigned char
#define ulong unsigned long
#define uint unsigned int
#define bool uchar
#define true 1
#define false 0

/* Prototypes */
void StringCopy(uchar**, uchar*);
void AddToString(uchar**, uchar*);
ulong FileSize(FILE*);
uchar ReadChar(FILE*, ulong);
void WriteChar(FILE*, ulong, uchar);
void ToString(FILE*, ulong, uchar*);
ulong WhereIs(FILE*, uchar*, ulong);
uint StringLength(uchar*);
bool IsLegal(uchar);
bool SaveSymbolsToArray(uchar*, uchar**, uchar*);
bool IsUndefined(FILE*, ulong);
bool SaveArrayToFile(uchar*, uchar**, uchar);
bool ReplaceAt(FILE*, uchar*, ulong);

void StringCopy(uchar** ucppDest, uchar* ucpSource)
{
  uchar i=0;
  uchar *ucpTemp=*ucppDest;

  free(ucpTemp);
  ucpTemp=NULL;

  ucpTemp=(uchar*) malloc (StringLength(ucpSource)*sizeof(uchar) );
  

  if (ucpTemp==NULL)
  {
    printf("Error! Couldn't alloc memory in function StringCopy!\n");
    exit(EXIT_FAILURE);
  }

  while (IsLegal(ucpSource[i]))
  {
    ucpTemp[i]=ucpSource[i];
    i++;
  }
  ucpTemp[i]=0x00;
  *ucppDest=ucpTemp;
}

void AddToString(uchar** ucpString, uchar* ucpToAdd)
{
  uchar *ucpTemp=*ucpString;
  uchar *ucpOld=*ucpString;
  uchar ucFrom=StringLength(ucpTemp);
  uchar ucLength=StringLength(ucpToAdd);
  uchar i;

  ucpTemp=(uchar*) malloc( (StringLength(ucpOld)+
                            StringLength(ucpToAdd))*
                            sizeof(uchar) );
  for (i=0; i<=ucFrom; i++)
    ucpTemp[i]=ucpOld[i];

  free(ucpOld);
  ucpOld=NULL;

  for (i=0; i<=ucLength; i++)
  {
    ucpTemp[ucFrom+i]=ucpToAdd[i];
  }
  ucpTemp[ucFrom+i+1]=0x00;
  *ucpString=ucpTemp;
}

ulong FileSize(FILE* fpName)
{
  ulong ulTemp=0;
  fseek(fpName, 0, 0);
  while (!feof(fpName))
  {
    (void) fgetc(fpName);
    ulTemp++;
  }
  fseek(fpName,0, 0);
  return ulTemp-1;
}

uchar ReadChar(FILE* fpName, ulong ulOffset)
{
  uchar ucTemp;
  fseek(fpName, ulOffset, 0);
  ucTemp=fgetc(fpName);
  fseek(fpName, ulOffset, 0);
  return ucTemp;
}

void WriteChar(FILE* fpName, ulong ulOffset, uchar ucChar)
{
  fseek(fpName, ulOffset, 0);
  fputc(ucChar, fpName);
  fseek(fpName, ulOffset, 0);
}

void ToString(FILE* fpName, ulong ulOffset, uchar* ucpString)
{
  uchar i=0;

  while (IsLegal(ReadChar(fpName, ulOffset+i)))
  {
    ucpString[i]=ReadChar(fpName, ulOffset+i);
    i++;
  }
}

ulong WhereIs(FILE* fpName, uchar* ucpString,ulong ulSize)
{
  uchar ucMaxFound=0;
  uchar ucHowLong=StringLength(ucpString);
  uchar ucCharsMatched;
  ulong ulOffset=0;
  bool found=false;

  printf(" Need %d Chars matching...", ucHowLong);
  while (ulOffset<=ulSize && found==false)
  {
    ucCharsMatched=0;
    if (ReadChar(fpName, ulOffset)==ucpString[ucCharsMatched])
    {
      while ( (ReadChar(fpName, ulOffset+ucCharsMatched)==
               ucpString[ucCharsMatched]) &&
              (ucCharsMatched<ucHowLong) )

      {
        ucCharsMatched++;
      }
      if (ucCharsMatched>ucMaxFound)
        ucMaxFound=ucCharsMatched;
      if (ucCharsMatched==ucHowLong)
        found=true;
    }
    ulOffset++;
  }
  printf("found max %d Chars\n", ucMaxFound);
  if (found) return (ulOffset);

  return (-1);
}

uint StringLength(uchar* ucpString)
{
  uchar i=0;
  while (IsLegal(ucpString[i]))
    i++;
  return i;                  
}

bool IsLegal(uchar cChar)
{
  if (cChar==0x00)
    return false;

  if (    (cChar>='a' && cChar<='z') ||
          (cChar>='A' && cChar<='Z') ||
          (cChar>='0' && cChar<='9') ||
          (cChar=='_') ||
          (cChar=='@') ||
          (cChar=='.')   )
    return true;
  else
    return false;
}

bool SaveSymbolsToArray(uchar* ucpFileName, uchar** ucppStringArray,
                        uchar* ucpNumber)
{
  FILE *fpFile;
  uchar ucCounter=0;
  ulong ulOffset=0;
  ulong ulSize;
  
  printf("Trying to open %s\n", ucpFileName);
  fpFile=fopen( (const char*) ucpFileName,"rb");
  if (fpFile==NULL)
  {
    printf("Error! File %s not found!\n", ucpFileName);
    return false;
  }

  printf("Opening %s\n", ucpFileName);
  ulSize=FileSize(fpFile);

  while (ulOffset<=ulSize)
  {
    if (IsUndefined(fpFile, ulOffset))
    {
      ToString(fpFile, ulOffset+17, ucppStringArray[ucCounter]);
      printf("Found %s\n", ucppStringArray[ucCounter]);
      ucCounter++;
    }
  ulOffset++;
  }
  if (ucCounter==0)
  {
    printf("For what do you use me???\nThere are no undefined symbols \n");
    printf("in %s !!!\n", ucpFileName);
    return false;
  }
  printf("Found %d undefined symbols!\n", ucCounter);
  fclose(fpFile);
  *ucpNumber=ucCounter;
  return true;
}

bool IsUndefined(FILE *fpFile, ulong ulOffset)
{
  if (  (ReadChar(fpFile, ulOffset+0)=='S') &&
        (ReadChar(fpFile, ulOffset+1)=='y') &&
        (ReadChar(fpFile, ulOffset+2)=='m') &&
        (ReadChar(fpFile, ulOffset+3)=='b') &&
        (ReadChar(fpFile, ulOffset+4)=='o') &&
        (ReadChar(fpFile, ulOffset+5)=='l') &&
        (ReadChar(fpFile, ulOffset+6)==' ') &&
        (ReadChar(fpFile, ulOffset+7)=='U') &&
        (ReadChar(fpFile, ulOffset+8)=='n') &&
        (ReadChar(fpFile, ulOffset+9)=='d') &&
        (ReadChar(fpFile, ulOffset+10)=='e') &&
        (ReadChar(fpFile, ulOffset+11)=='f') &&
        (ReadChar(fpFile, ulOffset+12)=='i') &&
        (ReadChar(fpFile, ulOffset+13)=='n') &&
        (ReadChar(fpFile, ulOffset+14)=='e') &&
        (ReadChar(fpFile, ulOffset+15)=='d') &&
        (ReadChar(fpFile, ulOffset+16)==' ') )
    return true;
  else
    return false;
}

bool SaveArrayToFile(uchar* ucpFile, uchar** ucppStringArray, uchar ucNumber)
{
  FILE *fpFile;
  uchar i;
  ulong ulFileSize;
  uchar ucCounter=ucNumber;  

  fpFile=fopen( (const char*) ucpFile, "rb+");
  if (fpFile==NULL)
  {
    printf("Error! File %s not found!\n", ucpFile);
    return false;
  }

  ulFileSize=FileSize(fpFile);

  for (i=0; i<ucNumber; i++)
  {
    if (!ReplaceAt(fpFile, ucppStringArray[i], ulFileSize))
    {
      printf(" ERROR at Processing symbol %s\n", ucppStringArray[i]);
    }
    else
    {
      ucCounter--;
    }
  }

  fclose(fpFile);
  if (ucCounter==0)
    return true;
  else
    return false;
}

bool ReplaceAt(FILE* fpFile, uchar* ucpString, ulong ulFileSize)
{
  ulong ulOffset;

  printf("Searching %s\n", ucpString);
  ulOffset=WhereIs(fpFile, ucpString, ulFileSize);

  if (ulOffset==-1)
  {
    printf("Symbol %s not found!\n", ucpString);
    return false;
  }

  while (ReadChar(fpFile, ulOffset)!='@')
    ulOffset++;

  WriteChar(fpFile, ulOffset, 0x00);
  ulOffset++;

  while ( (ReadChar(fpFile, ulOffset)>='0') &&
          (ReadChar(fpFile, ulOffset)<='9') )
  {
    WriteChar(fpFile, ulOffset, 0x00);
    ulOffset++;
  }

  printf(" Replaced symbol %s\n", ucpString);
  return true;
}
